import base64
from django.db import models
try:
    from django.utils.encoding import force_bytes
except:
    pass


class Base64Field(models.TextField):

    def contribute_to_class(self, cls, name):
        if self.db_column is None:
            self.db_column = name
        self.field_name = name + '_base64'
        super(Base64Field, self).contribute_to_class(cls, self.field_name)
        setattr(cls, name, property(self.get_data, self.set_data))

    def get_data(self, obj):
        try:
            content = base64.b64decode(
                force_bytes(
                    getattr(
                        obj,
                        self.field_name)))
            if isinstance(content, bytes):
                return content.decode("utf-8")
            else:
                return content
        except NameError:
            return base64.b64decode(getattr(obj, self.field_name))

    def set_data(self, obj, data):
        try:
            setattr(
                obj,
                self.field_name,
                base64.b64encode(force_bytes(data)))
        except NameError:
            setattr(obj, self.field_name, base64.b64encode(data))



try:
    from django.conf.urls import patterns, url
except ImportError:
    from django.conf.urls.defaults import patterns, url

try:
    from django.core.urlresolvers import reverse_lazy
except ImportError:
    from django.core.urlresolvers import reverse
    from django.utils.functional import lazy
    reverse_lazy = lambda *args, **kwargs: lazy(reverse, str)(*args, **kwargs)
from mcfeely.views import UnsubscribeView
from mcfeely.views import SuccessView

urlpatterns = patterns('mcfeely.views',
                       url(r'^success/$',
                           SuccessView.as_view(), name='success'),
                       url(r'^unsubscribe/$',
                           UnsubscribeView.as_view(
                           success_url=reverse_lazy(
                           'success')), name='unsubscribe', )
                       )



from django.views.generic.edit import CreateView
from django.views.generic import TemplateView
from mcfeely.forms import UnsubscribeForm


class UnsubscribeView(CreateView):
    form_class = UnsubscribeForm
    template_name = 'unsubscribe.html'


class SuccessView(TemplateView):
    template_name = 'success.html'



from django.core.mail.backends.base import BaseEmailBackend
from email.mime.base import MIMEBase

from mcfeely.models import Email, Attachment, Alternative, Header
from mcfeely.models import Recipient
from mcfeely.engine import default_queue


class DbBackend(BaseEmailBackend):

    def send_messages(self, email_messages):
        num_sent = 0

        if not email_messages:
            return num_sent

        for message in email_messages:

            if not hasattr(message, 'queue'):
                message.queue = default_queue(None)

            email = Email.objects.create(
                m_from=message.from_email,
                subject=message.subject,
                body=message.body,
                queue=message.queue
            )

            for address in message.to:
                Recipient.objects.create(
                    email=email,
                    address=address,
                    recipient_type='to')

            for address in message.cc:
                Recipient.objects.create(
                    email=email,
                    address=address,
                    recipient_type='cc')

            for address in message.bcc:
                Recipient.objects.create(
                    email=email,
                    address=address,
                    recipient_type='bcc')

            for attachment in message.attachments:
                if isinstance(attachment, tuple):
                    filename, content, mimetype = attachment
                elif isinstance(attachment, MIMEBase):
                    filename = attachment.get_filename()
                    content = attachment.get_payload(decode=True)
                    mimetype = "%s/%s" % (attachment.get_content_maintype(), attachment.get_content_subtype())
                else:
                    continue

                Attachment.objects.create(
                    email=email,
                    filename=filename,
                    content=content,
                    mimetype=mimetype
                )

            for header in message.extra_headers:
                Header.objects.create(
                    email=email,
                    key=header,
                    value=message.extra_headers[header],
                )

            try:
                for alternative in message.alternatives:
                    content, mimetype = alternative
                    Alternative.objects.create(
                        email=email,
                        content=content,
                        mimetype=mimetype
                    )
            except AttributeError:
                pass

            if(email):
                num_sent += 1

        return num_sent
